/*****************************************************************************/
/*                        WiNRADiO G39DDC EXCELSIOR                          */
/*                                SDK DEMO                                   */
/*             Receiver selection and "No receiver found" dialogs            */
/*                        (C) Copyright RADIXON 2014                         */
/*****************************************************************************/

#include "u_receiverselector.h"
#include "u_defs.h"
#include "u_global.h"

extern G39DDCAPI_CREATE_INSTANCE CreateInstance;

TNoReceiverFoundWindow::TNoReceiverFoundWindow(void):wxDialog(NULL,wxID_ANY,APP_TITLE,wxDefaultPosition)
{
 wxStaticText *t;
 wxSizer *Sizer,*ButtonSizer;

    SetFont(*Global.m_Font);

    Sizer=new wxBoxSizer(wxVERTICAL);

    t=new wxStaticText(this,wxID_ANY,wxT("No G39DDC receiver has been found.\n\n")
                                  wxT("This may mean that either no receiver is connected,\n")
                                  wxT("or it is not powered, or its driver was not\n")
                                  wxT("loaded in time before this application was executed.\n\n")
                                  wxT("In the demo mode, a simulated 15 MHz signal,\n")
                                  wxT("amplitude-modulated with 1 kHz tone, is provided."),
                    wxDefaultPosition,wxDefaultSize,wxALIGN_CENTRE);


    Sizer->Add(t,1,wxALIGN_TOP|wxALIGN_CENTRE|wxTOP,20);

    ButtonSizer=new wxBoxSizer(wxHORIZONTAL);

    Sizer->Add(ButtonSizer,0,wxALIGN_CENTER|wxALIGN_TOP|wxBOTTOM,20);

    m_BTryAgain=new wxButton(this,wxID_RETRY,wxT("Try again"),wxDefaultPosition,wxSize(65,23));
    ButtonSizer->Add(m_BTryAgain,0,wxALIGN_CENTER);

    ButtonSizer->AddSpacer(4);

    m_BDemo=new wxButton(this,wxID_DEFAULT,wxT("Run in demo mode"),wxDefaultPosition,wxSize(110,23));
    ButtonSizer->Add(m_BDemo,0,wxALIGN_CENTER);

    ButtonSizer->AddSpacer(4);

    m_BCancel=new wxButton(this,wxID_CANCEL,wxT("Cancel"),wxDefaultPosition,wxSize(65,23));
    ButtonSizer->Add(m_BCancel,0,wxALIGN_CENTER);

    SetSizer(Sizer);

    SetClientSize(300,193);

    Layout();

    CenterOnScreen();
}

TNoReceiverFoundWindow::~TNoReceiverFoundWindow(void)
{
}

void TNoReceiverFoundWindow::OnTryAgain(wxCommandEvent &Event)
{
    EndModal(wxID_RETRY);
}

void TNoReceiverFoundWindow::OnDemo(wxCommandEvent &Event)
{
    EndModal(wxID_DEFAULT);
}

BEGIN_EVENT_TABLE(TNoReceiverFoundWindow, wxDialog)
    EVT_BUTTON(wxID_RETRY,TNoReceiverFoundWindow::OnTryAgain)
    EVT_BUTTON(wxID_DEFAULT,TNoReceiverFoundWindow::OnDemo)
END_EVENT_TABLE()

TReceiverSelector::TReceiverSelector(IG39DDCDeviceEnumerator *Enumerator):wxDialog(NULL,wxID_ANY,APP_TITLE,wxDefaultPosition)
{
 uint32_t i;
 G39DDC_DEVICE_INFO DeviceInfo;
 wxString *list;
 wxPanel *p;

    SetFont(*Global.m_Font);

    p=new wxPanel(this,0,0,100,100);

    Layout();

    list=new wxString[Enumerator->GetCount()];

    for(i=0;i<Enumerator->GetCount();i++)
    {
        Enumerator->GetDeviceInfo(i,&DeviceInfo,sizeof(DeviceInfo));

        list[i].sprintf(wxT("%s  %s"),wxString(DeviceInfo.SerialNumber,wxConvLocal).c_str(),DeviceInfo.InterfaceType==G39DDC_INTERFACE_TYPE_PCIE?wxT("PCIe"):wxT("USB"));
    }

    m_RadioBox=new wxRadioBox(p,wxID_ANY,wxT("Select receiver to open"),wxPoint(20,20),wxSize(130,Enumerator->GetCount()*20+25),Enumerator->GetCount(),list,0,wxVERTICAL);

    delete[] list;

    m_BOK=new wxButton(p,wxID_OK,wxT("OK"),wxPoint(165,20),wxSize(70,23));

    m_BCancel=new wxButton(p,wxID_CANCEL,wxT("Cancel"),wxPoint(165,50),wxSize(70,23));

    m_SelectedReceiverIndex=m_RadioBox->GetSelection();

    SetClientSize(m_BOK->GetRect().GetRight()+20,m_RadioBox->GetRect().GetBottom()+20);

    CenterOnScreen();
}

TReceiverSelector::~TReceiverSelector(void)
{
}

void TReceiverSelector::OnOk(wxCommandEvent &Event)
{
    m_SelectedReceiverIndex=m_RadioBox->GetSelection();
    EndModal(wxID_OK);
}

BEGIN_EVENT_TABLE(TReceiverSelector, wxDialog)
    EVT_BUTTON(wxID_OK,TReceiverSelector::OnOk)
END_EVENT_TABLE()


TReceiver* TReceiverSelector::SelectReceiver(void)
{
 IG39DDCDeviceEnumerator *Enumerator;
 TNoReceiverFoundWindow *NoReceiver;
 TReceiver *Receiver;
 TReceiverSelector *Selector;
 uint32_t Count;
 int r;
 G39DDC_DEVICE_INFO DeviceInfo;

    if(!CreateInstance(G39DDC_CLASS_ID_DEVICE_ENUMERATOR,(void**)&Enumerator))
    {
        wxMessageBox(wxT("Failed to create device enumerator object!"),APP_TITLE,wxICON_ERROR);
        return NULL;
    }

    Receiver=new TReceiver;

    for(;;)
    {
        Count=Enumerator->GetCount();

        if(Count==0) //no receiver has been found
        {
            NoReceiver=new TNoReceiverFoundWindow;

            r=NoReceiver->ShowModal();

            delete NoReceiver;

            if(r==wxID_RETRY)
            {
                Enumerator->Enumerate();
            }
            else if(r==wxID_DEFAULT)
            {
                if(Receiver->Open(G39DDC_OPEN_DEMO))
                {
                    Enumerator->Release();
                    return Receiver;
                }

                wxMessageBox(wxT("Failed to open demo receiver!"),APP_TITLE,wxICON_ERROR);
                break;
            }
            else
            {
                break;
            }
        }
        else if(Count==1) //single receiver has been found
        {
            Enumerator->GetDeviceInfo(0,&DeviceInfo,sizeof(DeviceInfo));

            if(Receiver->Open(DeviceInfo.DevicePath))
            {
                //receiver has been open
                Enumerator->Release();
                return Receiver;
            }

            wxMessageBox(wxT("Failed to open receiver!"),APP_TITLE,wxICON_ERROR);
            break;
        }
        else
        {
            //more than one receiver has been detected
            Selector=new TReceiverSelector(Enumerator);

            if(Selector->ShowModal()!=wxID_OK)
            {
                delete Selector;
                break;
            }

            r=Selector->GetSelectedReceiverIndex();

            delete Selector;

            Enumerator->GetDeviceInfo(r,&DeviceInfo,sizeof(DeviceInfo));

            if(Receiver->Open(DeviceInfo.DevicePath))
            {
                Enumerator->Release();
                return Receiver;
            }

            wxMessageBox(wxT("Failed to open receiver!"),APP_TITLE,wxICON_ERROR);
            break;
        }
    }

    Enumerator->Release();
    delete Receiver;

    return NULL;
}
